﻿using Infragistics.Controls.Grids;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Wpf3DChart
{
    /// <summary>
    /// Interaction logic for XamGridWindow.xaml
    /// </summary>
    public partial class XamGridWindow : Window
    {
        public XamGridWindow()
        {
            InitializeComponent();
            SetData();
        }

        public void SetData()
        {

            grid.ItemsSource = null;
            grid.AutoGenerateColumns = false;
            grid.SortingSettings.AllowSorting = false;
            grid.UpdateLayout();

            TextColumn rowIndexCol = new TextColumn();
            rowIndexCol.Key = "LineIndex";
            rowIndexCol.HeaderText = " ";
            grid.Columns.Add(rowIndexCol);

            TextColumn timeCol = new TextColumn();
            timeCol.Key = "StampTime";
            timeCol.HeaderText = "Time";
            timeCol.FormatString = "{0: " + "HH:mm:ss.fff" + "}";
            grid.Columns.Add(timeCol);

            TextColumn lineCol = new TextColumn();
            lineCol.Key = "Line";
            lineCol.ValueConverterParameter = 0;
            lineCol.Width = new ColumnWidth(1, true);

            grid.Columns.Add(lineCol);

            foreach (EditableColumn column in grid.Columns)
            {
                column.Width = ColumnWidth.Auto;
            }

            AddConditional(lineCol);


            grid.ItemsSource = LoadData();
        }

        private void AddConditional(TextColumn col)
        {
            LineConditionalFormatRule cond = new LineConditionalFormatRule();
            cond.Proxy.Style = (Style)grid.Resources["gridStyle"];
            col.ConditionalFormatCollection.Add(cond);
        }

        private LogLineInfoTest[] LoadData()
        {
            Random rand = new Random((int)DateTime.Now.Ticks);
            List<LogLineInfoTest> lines = new List<LogLineInfoTest>();
            for (int i = 0; i < 10000; i++)
            {
                string line = "Line number " + i;
                line = line.PadRight(rand.Next(0, 2000), '*');
                lines.Add(new LogLineInfoTest() { LineIndex = i, StampTime = DateTime.Now.AddSeconds(i * 5), Line = line });
                if (i % 10 == 0)
                    lines.Last().Foreground = "Red";
                if (i % 13 == 0)
                    lines.Last().Foreground = "Blue";
                if (i % 17 == 0)
                    lines.Last().Foreground = "Orange";
            }

            return lines.ToArray();

        }

        private void CellControl_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            CellControl cell = sender as CellControl;
            Row row = cell.Cell.Row as Row;
            row.IsSelected = true;

            e.Handled = true;
        }






    }

    /// <summary>
    /// 
    /// </summary>
    public class LineConditionalFormatRule : ConditionalFormattingRuleBase
    {
        /// <summary>
        /// 
        /// </summary>
        public LineConditionalFormatRuleProxy Proxy { get; set; }



        /// <summary>
        /// 
        /// </summary>
        public LineConditionalFormatRule()
        {
            Proxy = new LineConditionalFormatRuleProxy();
        }


        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        protected override IConditionalFormattingRuleProxy CreateProxy()
        {
            return Proxy;
        }

    }


    /// <summary>
    /// 
    /// </summary>
    public class LineConditionalFormatRuleProxy : ConditionalFormattingRuleBaseProxy
    {
        public Style Style { get; set; }

        /// <summary>
        /// 
        /// </summary>
        public LineConditionalFormatRuleProxy()
        {

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sourceDataObject"></param>
        /// <param name="sourceDataValue"></param>
        /// <returns></returns>
        protected override Style EvaluateCondition(object sourceDataObject, object sourceDataValue)
        {
            LogLineInfoTest lvli = sourceDataObject as LogLineInfoTest;
            return CreateStyleFromDisplayAttributes(lvli);
        }

        private Style CreateStyleFromDisplayAttributes(LogLineInfoTest lvli)
        {
            Style style = new Style(typeof(ConditionalFormattingCellControl));
          style.BasedOn = Style;
            
            style.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.ForegroundProperty, Value = new SolidColorBrush((Color)ColorConverter.ConvertFromString(lvli.Foreground)) });
            
            return style;
        }

        private void HandleClick(object sender, RoutedEventArgs e)
        {
            CellControl cell = sender as CellControl;
            Row row = cell.Cell.Row as Row;
            row.IsSelected = true;

            e.Handled = true;

        }



    }


    public class LogLineInfoTest
    {
        /// <summary>
        /// 
        /// </summary>
        public LogLineInfoTest()
        {


        }



        /// <summary>
        /// 
        /// </summary>
        public int? LineIndex { get; set; }

        /// <summary>
        /// 
        /// </summary>
        public String Line { get; set; }

        /// <summary>
        /// 
        /// </summary>
        public DateTime StampTime { get; set; }

        public string Foreground { get; set; } = "Black";

    }
}
